package cn.fw.valhalla.rpc.erp;

import cn.fw.data.base.domain.common.Message;
import cn.fw.erp.sdk.api.UserApi;
import cn.fw.erp.sdk.api.UserRoleApi;
import cn.fw.erp.sdk.api.result.UserBaseInfo;
import cn.fw.erp.sdk.api.result.UserRoleDataRange;
import cn.fw.erp.sdk.api.result.UserRoleInfo;
import cn.fw.valhalla.common.utils.StringUtils;
import cn.fw.valhalla.rpc.erp.dto.PostUserDTO;
import cn.fw.valhalla.rpc.erp.dto.UserInfoDTO;
import cn.fw.valhalla.rpc.erp.dto.UserRoleDataRangeDTO;
import com.alibaba.fastjson.JSONObject;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.BoundListOperations;
import org.springframework.data.redis.core.BoundValueOperations;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import static org.apache.commons.lang3.Validate.isTrue;
import static org.apache.commons.lang3.Validate.notNull;

/**
 * 用户服务
 * <p>
 * create at 2019-03-09
 *
 * @author kurisu
 */
@Slf4j
@Service
public class UserService {
    /**
     * 用户服务
     */
    private final UserApi userApi;
    /**
     * 岗位用户服务
     */
    private final UserRoleApi userRoleApi;
    /**
     * Redis工具
     */
    private final StringRedisTemplate redisTemplate;

    @Value("${spring.cache.custom.global-prefix}:rpc:erp")
    @Getter
    private String keyPrefix;


    /**
     * 默认构造器
     */
    @Autowired
    public UserService(final UserApi userApi,
                       final UserRoleApi userRoleApi,
                       final StringRedisTemplate redisTemplate) {
        this.userApi = userApi;
        this.userRoleApi = userRoleApi;
        this.redisTemplate = redisTemplate;
    }

    /**
     * 根据用户ID获取用户信息
     *
     * @param userId 用户ID
     * @return 用户信息
     */
    public UserInfoDTO user(final Long userId) {
        if (Objects.isNull(userId)) {
            return null;
        }
        String key = generateKey(userId);
        String json = getUserFromCache(key);
        if (StringUtils.isValid(json)) {
            return JSONObject.parseObject(json, UserInfoDTO.class);
        }
        try {
            final Message<UserBaseInfo> msg = userApi.queryUserBaseInfo(userId);
            isTrue(msg.isSuccess(), String.format("调用ERP[根据用户ID[%s]获取用户信息]系统失败", msg.getResult()));
            final UserBaseInfo userBaseInfo = msg.getData();
            if (!msg.isSuccess() || Objects.isNull(msg.getData())) {
                return null;
            }
            UserInfoDTO userInfoDTO = new UserInfoDTO();
            userInfoDTO.setId(userBaseInfo.getUserId());
            userInfoDTO.setUserName(userBaseInfo.getUserName());
            userInfoDTO.setHeadImg(userBaseInfo.getHeadImg());
            userInfoDTO.setMobile(msg.getData().getMobile());
            setUserToCache(key, JSONObject.toJSONString(userInfoDTO));
            return userInfoDTO;
        } catch (Exception e) {
            log.info("调用ERP[根据用户ID[{}]获取用户信息]系统失败", userId, e);
        }
        return null;
    }

    /**
     * 批量获取用户名字
     *
     * @param userIds 用户ID集合
     * @return 用户姓名集合
     */
    public List<String> userNames(final List<Long> userIds) {
        if (CollectionUtils.isEmpty(userIds)) {
            return Collections.emptyList();
        }
        Message<List<UserBaseInfo>> userMsg = userApi.queryUserBaseInfos(userIds);
        isTrue(userMsg.isSuccess(), "批量获取用户信息失败");
        notNull(userMsg.getData(), "批量获取用户信息为空");
        return userMsg.getData().stream().map(UserBaseInfo::getUserName).collect(Collectors.toList());
    }

    /**
     * 批量获取用户
     *
     * @param userIds 用户ID集合
     * @return 用户集合
     */
    public List<UserInfoDTO> users(final List<Long> userIds) {
        if (CollectionUtils.isEmpty(userIds)) {
            return Collections.emptyList();
        }
        Message<List<UserBaseInfo>> userMsg = userApi.queryUserBaseInfos(userIds);
        isTrue(userMsg.isSuccess(), "批量获取用户信息失败");
        notNull(userMsg.getData(), "批量获取用户信息为空");
        List<UserInfoDTO> result = new ArrayList<>();
        List<UserBaseInfo> userBaseInfoList = userMsg.getData();
        for (UserBaseInfo userBaseInfo : userBaseInfoList) {
            UserInfoDTO userInfoDTO = new UserInfoDTO();
            userInfoDTO.setId(userBaseInfo.getUserId());
            userInfoDTO.setUserName(userBaseInfo.getUserName());
            userInfoDTO.setHeadImg(userBaseInfo.getHeadImg());
            userInfoDTO.setMobile(userBaseInfo.getMobile());
            result.add(userInfoDTO);
        }
        return result;
    }

    /**
     * 获取某个流程角色的权限范围
     *
     * @param userId
     * @param roleCode
     * @return
     */
    public List<UserRoleDataRangeDTO> getUserRoleDataRange(final Long userId, final String roleCode) {
        if (Objects.isNull(userId) || Objects.isNull(roleCode)) {
            return Collections.emptyList();
        }
        try {
            Message<List<UserRoleDataRange>> msg = userRoleApi.queryUserRoleDataRange(userId, roleCode);
            log.info("userRoleApi.queryUserRoleDataRange: msg.code={}, msg.result={}, msg.data={}", msg.getCode(), msg.getResult(), msg.getData());
            if (!msg.isSuccess() || CollectionUtils.isEmpty(msg.getData())) {
                return Collections.emptyList();
            }
            List<UserRoleDataRange> data = msg.getData();
            List<UserRoleDataRangeDTO> list = new ArrayList<>(data.size());
            for (UserRoleDataRange datum : data) {
                UserRoleDataRangeDTO dto = new UserRoleDataRangeDTO();
                BeanUtils.copyProperties(datum, dto);
                list.add(dto);
            }
            return list;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return Collections.emptyList();
    }

    /**
     * 获取门店下某岗位下员工
     *
     * @param shopId   门店
     * @param roleCode 岗位码
     * @return 用户集合
     */
    public List<PostUserDTO> getUserByRole(final Long shopId, final String roleCode) {
        if (Objects.isNull(shopId) || Objects.isNull(roleCode)) {
            return Collections.emptyList();
        }
        String key = generateKey(shopId, roleCode);
        List<PostUserDTO> dtos = getFromCache(key);
        if (!CollectionUtils.isEmpty(dtos)) {
            return dtos;
        }
        try {
            Message<List<UserRoleInfo>> msg = userRoleApi.queryUserRoleInfos(roleCode, shopId);
            log.info("userRoleApi.queryUserRoleInfos: msg.code={}, msg.result={}, msg.data={}", msg.getCode(), msg.getResult(), msg.getData());
            if (!msg.isSuccess() || CollectionUtils.isEmpty(msg.getData())) {
                return Collections.emptyList();
            }
            List<UserRoleInfo> data = msg.getData();
            List<PostUserDTO> list = new ArrayList<>(data.size());
            for (UserRoleInfo datum : data) {
                PostUserDTO dto = new PostUserDTO();
                BeanUtils.copyProperties(datum, dto);
                list.add(dto);
                setToCache(key, JSONObject.toJSONString(dto));
            }
            getQueue(key).expire(10, TimeUnit.MINUTES);
            return list;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return Collections.emptyList();
    }


    private List<PostUserDTO> getFromCache(final String key) {
        try {
            BoundListOperations<String, String> queue = getQueue(key);
            final long size = Optional.ofNullable(queue.size()).orElse(0L);
            if (size > 0) {
                final List<PostUserDTO> userDTOS = new ArrayList<>();
                for (long i = 0; i < size; i++) {
                    final String json = Objects.requireNonNull(queue.index(i));
                    userDTOS.add(JSONObject.parseObject(json, PostUserDTO.class));
                }
                return userDTOS;
            }
        } catch (Exception e) {
            log.error("erpRpc  从缓存获取用户信息失败", e);
        }
        return null;
    }

    private void setToCache(final String key, final String value) {
        try {
            getQueue(key).rightPush(value);
        } catch (Exception e) {
            log.error("erpRpc  缓存用户信息失败", e);
        }
    }

    private BoundListOperations<String, String> getQueue(final String key) {
        return redisTemplate.boundListOps(key);
    }


    private String generateKey(final Long shopId, final String roleCOde) {
        Assert.notNull(shopId, "id cannot be null");
        Assert.hasLength(roleCOde, "roleCOde cannot be null");

        return String.format("%s:%s:%s:%s", getKeyPrefix(), "post", shopId, roleCOde);
    }

    private String generateKey(final Long userId) {
        Assert.notNull(userId, "userId cannot be null");

        return String.format("%s:%s:%s", getKeyPrefix(), "user", userId);
    }

    private String getUserFromCache(final String key) {
        String json = null;
        try {
            BoundValueOperations<String, String> ops = redisTemplate.boundValueOps(key);
            json = ops.get();
        } catch (Exception e) {
            log.error("erpRpc  从缓存获取用户信息失败", e);
        }
        return json;
    }


    private void setUserToCache(final String key, final String value) {
        try {
            redisTemplate.opsForValue().set(key, value, 30, TimeUnit.SECONDS);
        } catch (Exception e) {
            log.error("erpRpc  从缓存获取用户信息失败", e);
        }
    }
}
