package cn.fw.valhalla.rpc.oop;

import cn.fw.data.base.domain.common.Message;
import cn.fw.oop.sdk.*;
import cn.fw.oop.sdk.enums.BizTypeEnum;
import cn.fw.oop.sdk.result.*;
import cn.fw.valhalla.common.utils.StringUtils;
import cn.fw.valhalla.rpc.oop.dto.*;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.BoundValueOperations;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * OOP系统
 * <p>
 * create at 2019-03-07
 *
 * @author kurisu
 */
@Slf4j
@Service
@SuppressWarnings("Duplicates")
public class OopService {

    /**
     * 门店服务
     */
    private final ShopApiService shopApiService;
    /**
     * 商家服务
     */
    private final DealerApiService dealerApiService;
    /**
     * 品牌服务
     */
    private final CarBrandApiService carBrandApiService;
    /**
     * 车系服务
     */
    private final CarSeriesApiService carSeriesApiService;
    /**
     * 车型服务
     */
    private final CarSpecApiService carSpecApiService;


    private final RegionApiService regionApiService;

    /**
     * 集团服务
     */
    private final GroupApiService groupApiService;
    /**
     * Redis工具
     */
    private final StringRedisTemplate redisTemplate;

    @Value("${spring.cache.custom.global-prefix}:rpc:oop")
    @Getter
    private String keyPrefix;

    /**
     * 默认构造器
     */
    @Autowired
    public OopService(final ShopApiService shopApiService,
                      final DealerApiService dealerApiService,
                      final CarBrandApiService carBrandApiService,
                      final CarSeriesApiService carSeriesApiService,
                      final CarSpecApiService carSpecApiService,
                      final GroupApiService groupApiService,
                      final RegionApiService regionApiService,
                      final StringRedisTemplate redisTemplate) {
        this.shopApiService = shopApiService;
        this.dealerApiService = dealerApiService;
        this.carBrandApiService = carBrandApiService;
        this.carSeriesApiService = carSeriesApiService;
        this.carSpecApiService = carSpecApiService;
        this.groupApiService = groupApiService;
        this.regionApiService = regionApiService;
        this.redisTemplate = redisTemplate;
    }

    /**
     * 根据车系ID获取车系信息
     *
     * @param seriesId 车系ID
     * @return 车系信息
     */
    public SeriesDTO getSeriesById(final Long seriesId) {
        if (seriesId == null) {
            return null;
        }
        String key = generateKey(OopTypeEnum.SERIES, seriesId);
        String json = getFromCache(key);
        if (StringUtils.isValid(json)) {
            return JSONObject.parseObject(json, SeriesDTO.class);
        }
        try {
            final Message<CarSeriesInfoVo> msg = carSeriesApiService.getSeriesInfo(seriesId);
            log.info("carSeriesApiService.getSeriesInfo: msg.code={}, msg.result={}, msg.data={}", msg.getCode(), msg.getResult(), msg.getData());
            if (msg.isSuccess() && !Objects.isNull(msg.getData())) {
                final SeriesDTO series = new SeriesDTO();
                BeanUtils.copyProperties(msg.getData(), series);
                setToCache(key, JSONObject.toJSONString(series));
                return series;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 根据品牌ID获取品牌信息
     *
     * @param brandId 品牌ID
     * @return 品牌信息
     */
    public BrandDTO getBrandById(final Long brandId) {
        if (brandId == null) {
            return null;
        }
        String key = generateKey(OopTypeEnum.BRAND, brandId);
        String json = getFromCache(key);
        if (StringUtils.isValid(json)) {
            return JSONObject.parseObject(json, BrandDTO.class);
        }
        try {
            final Message<CarBrandInfoVo> msg = carBrandApiService.getBrandInfo(brandId);
            log.info("carBrandApiService.getBrandInfo: msg.code={}, msg.result={}, msg.data={}", msg.getCode(), msg.getResult(), msg.getData());
            if (msg.isSuccess() && !Objects.isNull(msg.getData())) {
                final BrandDTO brand = new BrandDTO();
                BeanUtils.copyProperties(msg.getData(), brand);
                setToCache(key, JSONObject.toJSONString(brand));
                return brand;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 根据ID获取门店信息
     *
     * @param shopId 门店ID
     * @return 门店信息
     */
    public ShopDTO shop(final Long shopId) {
        if (shopId == null) {
            return null;
        }
        String key = generateKey(OopTypeEnum.SHOP, shopId);
        String json = getFromCache(key);
        if (StringUtils.isValid(json)) {
            return JSONObject.parseObject(json, ShopDTO.class);
        }
        Message<ShopInfoVo> msg = shopApiService.getShopInfo(shopId);
        log.info("shopApiService.getShopInfo 返回结果: msg.code={}, msg.result={}, msg.data={}", msg.getCode(), msg.getResult(), msg.getData());
        if (msg.isSuccess() && null != msg.getData()) {
            final ShopDTO shop = new ShopDTO();
            BeanUtils.copyProperties(msg.getData(), shop);
            setToCache(key, JSONObject.toJSONString(shop));
            return shop;
        }
        return null;
    }

    /**
     * 根据ID集合获取门店信息集合
     *
     * @param shopIds 门店ID结婚
     * @return 门店信息集合
     */
    public List<ShopDTO> shops(final List<Long> shopIds) {
        if (CollectionUtils.isEmpty(shopIds)) {
            return Collections.emptyList();
        }
        try {
            Message<List<ShopInfoVo>> msg = shopApiService.getShopInfoList(shopIds);
            log.info("shopApiService.getShopInfoList 返回结果: param[{}] data[{}]", shopIds, JSON.toJSONString(msg));
            if (!msg.isSuccess() || CollectionUtils.isEmpty(msg.getData())) {
                return Collections.emptyList();
            }
            return msg.getData().stream().map(item -> {
                final ShopDTO shop = new ShopDTO();
                BeanUtils.copyProperties(item, shop);
                return shop;
            }).collect(Collectors.toList());
        } catch (Exception e) {
            e.printStackTrace();
        }
        return Collections.emptyList();
    }

    /**
     * 根据ID获取商家信息
     *
     * @param dealerId 商家ID
     * @return 商家信息
     */
    public DealerDTO dealer(final Long dealerId) {
        if (dealerId == null) {
            return null;
        }
        String key = generateKey(OopTypeEnum.DEALER, dealerId);
        String json = getFromCache(key);
        if (StringUtils.isValid(json)) {
            return JSONObject.parseObject(json, DealerDTO.class);
        }
        Message<DealerInfoVo> msg = dealerApiService.getDealerInfo(dealerId);
        log.info("dealerApiService.getDealerInfo: msg.code={}, msg.result={}, msg.data={}", msg.getCode(), msg.getResult(), msg.getData());
        if (msg.isSuccess() && null != msg.getData()) {
            final DealerDTO dealer = new DealerDTO();
            BeanUtils.copyProperties(msg.getData(), dealer);
            setToCache(key, JSONObject.toJSONString(dealer));
            return dealer;
        }
        return null;
    }

    public List<DealerDTO> dealerDTOS(final List<Long> dealerIds) {
        if (CollectionUtils.isNotEmpty(dealerIds)) {
            Message<List<DealerInfoVo>> msg = dealerApiService.getDealerInfoList(dealerIds);
            BV.isTrue(msg.isSuccess(), String.format("调用OOP系统[根据ID[%s]获取商家对象]失败", dealerIds));
            List<DealerDTO> dtos = new ArrayList<>();
            msg.getData().forEach(r -> {
                final DealerDTO dealer = new DealerDTO();
                BeanUtils.copyProperties(r, dealer);
                dtos.add(dealer);
            });
            return dtos;
        } else {
            return new ArrayList<>();
        }
    }

    public List<DealerDTO> dealers(final Long groupId) {
        if (groupId == null) {
            return null;
        }
        final Message<List<DealerInfoVo>> msg = dealerApiService.getDealerInfoList(groupId);
        if (msg.isSuccess() && msg.getData() != null) {
            final List<DealerInfoVo> infos = msg.getData();
            return infos.stream().map(item -> {
                final DealerDTO dealer = new DealerDTO();
                BeanUtils.copyProperties(item, dealer);
                return dealer;
            }).collect(Collectors.toList());
        } else {
            return null;
        }

    }

    public List<DealerDTO> dealersByBrand(final Long groupId, final Long brandId) {
        final Message<List<DealerInfoVo>> msg = dealerApiService.getDealerInfoList(groupId, brandId);
        BV.isTrue(msg.isSuccess(), String.format("调用OOP系统，根据集团ID[%d]获取商家对象失败", groupId));
        final List<DealerInfoVo> infos = msg.getData();
        BV.isTrue(CollectionUtils.isNotEmpty(infos), String.format("集团[%d]下没有经营品牌[%d]的商家", groupId, brandId));
        return infos.stream().map(item -> {
            final DealerDTO dealer = new DealerDTO();
            BeanUtils.copyProperties(item, dealer);
            return dealer;
        }).collect(Collectors.toList());
    }

    public SpecDTO spec(final Long specId) {
        if (Objects.isNull(specId)) {
            return null;
        }
        String key = generateKey(OopTypeEnum.SPEC, specId);
        String json = getFromCache(key);
        if (StringUtils.isValid(json)) {
            return JSONObject.parseObject(json, SpecDTO.class);
        }
        try {
            Message<CarSpecInfoVo> msg = carSpecApiService.getSpecInfo(specId);
            log.info("carSpecApiService.getSpecInfo: msg.code={}, msg.result={}, msg.data={}", msg.getCode(), msg.getResult(), msg.getData());
            if (msg.isSuccess() && !Objects.isNull(msg.getData())) {
                final CarSpecInfoVo dto = msg.getData();
                SpecDTO spec = new SpecDTO();
                BeanUtils.copyProperties(dto, spec);
                return spec;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }


    public RegionDto getRegionByName(String name) {
        Message<RegionInfoVo> msg = regionApiService.getRegionInfoByName(name);
        if (msg.isSuccess() && msg.getData() != null) {
            log.info("根据区域名称查询区域返回结果:{}", msg.getData());
            final RegionInfoVo dto = msg.getData();
            RegionDto regionDto = new RegionDto();
            BeanUtils.copyProperties(dto, regionDto);
            return regionDto;
        } else {
            log.info("调用[oop系统regionApiService.getRegionByName]查询区域失败, param: {}, msg: code={}, result={}", name, msg.getCode(), msg.getResult());
            return null;
        }
    }

    /**
     * 获取集团商家级联树
     *
     * @return 集团商家级联树列表
     */
    public List<GroupTreeVo> getGroupDealerTree() {
        Message<List<GroupTreeVo>> msg = groupApiService.getGroupDealerTree();
        BV.isTrue(msg.isSuccess(), String.format("调用OOP系统获取集团商家级联树]失败"));
        if (Objects.nonNull(msg.getData())) {
            return msg.getData();
        } else {
            return new ArrayList<>();
        }
    }

    /**
     * 获取全部车辆品牌
     *
     * @return 品牌集合
     */
    @SuppressWarnings("Duplicates")
    public List<CarBrandInfoVo> getCarBrandAll() {
        String key = generateKey(OopTypeEnum.BRAND, "allBrand");
        String json = getFromCache(key);
        if (StringUtils.isValid(json)) {
            return JSONArray.parseArray(json, CarBrandInfoVo.class);
        }
        try {
            final Message<List<CarBrandInfoVo>> msg = carBrandApiService.getBrandAll();
            log.info("调用OOP系统获取[车辆全部品牌] data: [{}]", JSON.toJSONString(msg));
            List<CarBrandInfoVo> data = msg.getData();
            if (Objects.nonNull(data) && CollectionUtils.isNotEmpty(data)) {
                setToCache(key, JSONArray.toJSONString(data), 60 * 30);
                return data;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return Collections.emptyList();
    }

    /**
     * 根据品牌id获取车系列表
     *
     * @param brandId 品牌id
     * @return 车系列表
     */
    public List<CarSeriesInfoVo> getCarSeriesListByBrandId(final Long brandId) {
        if (Objects.isNull(brandId)) {
            return Collections.emptyList();
        }
        try {
            final Message<List<CarSeriesInfoVo>> msg = carSeriesApiService.getSeriesInfoList(brandId);
            log.info("调用OOP系统获取[根据品牌ID[{}]查询车系列表] 结果：[{}]", brandId, msg.getResult());
            if (!msg.isSuccess() || CollectionUtils.isEmpty(msg.getData())) {
                return Collections.emptyList();
            }
            return msg.getData();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return Collections.emptyList();
    }

    /**
     * 商家展厅
     *
     * @param dealerId 商家ID
     * @return 商家展厅
     */
    public ShopDTO dealerExhibitionHall(final Long dealerId) {
        Message<List<ShopInfoVo>> msg = shopApiService.getShopInfoList(null, dealerId, BizTypeEnum.AFTER_SALE.getValue());
        List<ShopInfoVo> data = msg.getData();
        if (msg.isSuccess() && data != null) {
            log.info("调用OOP系统获取商家[{}]展厅: data", data);
            final ShopDTO shop = new ShopDTO();
            BeanUtils.copyProperties(data, shop);
            return shop;
        } else {
            log.info("调用OOP系统获取商家[{}]展厅失败, message: code :{}, result:{}", dealerId, msg.getCode(), msg.getResult());
            return null;
        }
    }


    public GroupDTO group(final Long groupId) {
        String key = generateKey(OopTypeEnum.GROUP, groupId);
        String json = getFromCache(key);
        if (StringUtils.isValid(json)) {
            return JSONObject.parseObject(json, GroupDTO.class);
        }
        try {
            log.info("调用OP系统查询集团信息, groupId: {}", groupId);
            final Message<GroupInfoVo> msg = groupApiService.getGroupInfo(groupId);
            if (msg.isSuccess() && msg.getData() != null) {
                log.info("调用OP系统查询集团信息成功, result: {}", msg.getData());
                GroupDTO groupDTO = new GroupDTO();
                BeanUtils.copyProperties(msg.getData(), groupDTO);
                setToCache(key, JSONObject.toJSONString(groupDTO));
                return groupDTO;
            } else {
                log.error("调用OP系统查询集团信息失败, 原因: {}", msg.getResult());
            }
        } catch (Exception e) {
            log.warn("调用OP系统查询集团信息失败, specId: {}", groupId, e);
        }
        return null;
    }


    /**
     * 根据所有门店信息
     *
     * @return 门店信息列表
     */
    public List<ShopDTO> getAllShops(Long groupId) {
        Message<List<ShopInfoVo>> msg = shopApiService.getShopInfoList(groupId, null, null);
        if (msg.isSuccess() && null != msg.getData()) {
            return msg.getData().stream().map(shopInfo -> {
                final ShopDTO shopDTO = new ShopDTO();
                BeanUtils.copyProperties(shopInfo, shopDTO);
                return shopDTO;
            }).collect(Collectors.toList());
        }
        return new ArrayList<>();
    }

    private String getFromCache(final String key) {
        String json = null;
        try {
            BoundValueOperations<String, String> ops = redisTemplate.boundValueOps(key);
            json = ops.get();
        } catch (Exception e) {
            log.error("oopRpc  从缓存获取门店信息失败", e);
        }
        return json;
    }

    private void setToCache(final String key, final String value) {
        setToCache(key, value, 30);
    }

    private void setToCache(final String key, final String value, long timeout) {
        try {
            redisTemplate.opsForValue().set(key, value, timeout, TimeUnit.SECONDS);
        } catch (Exception e) {
            log.error("oopRpc  缓存门店信息失败", e);
        }
    }


    private String generateKey(final OopTypeEnum typeEnum, final Long id) {
        Assert.notNull(id, "id cannot be null");

        return String.format("%s:%s:%s", getKeyPrefix(), typeEnum.name(), id);
    }

    private String generateKey(final OopTypeEnum typeEnum, final String keyWord) {
        BV.isTrue(StringUtils.isValid(keyWord), "keyWord cannot be null");

        return String.format("%s:%s:%s", getKeyPrefix(), typeEnum.name(), keyWord);
    }

    enum OopTypeEnum {
        SHOP,
        DEALER,
        GROUP,
        BRAND,
        SERIES,
        SPEC,
        REGION
    }
}
