package cn.fw.valhalla.controller.task;

import cn.fw.valhalla.common.utils.DateUtil;
import cn.fw.valhalla.domain.db.follow.FollowRecord;
import cn.fw.valhalla.domain.enums.FollowTypeEnum;
import cn.fw.valhalla.rpc.erp.TodoRpcService;
import cn.fw.valhalla.rpc.erp.dto.BackLogItemDTO;
import cn.fw.valhalla.service.bus.follow.FollowBizService;
import cn.fw.valhalla.service.data.FollowRecordService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.Getter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

/**
 * @author : kurisu
 * @className : FollowRecordTask
 * @description : 跟进任务定时任务
 * @date: 2020-08-25 16:56
 */
@Component
@ConditionalOnProperty(prefix = "task", name = "switch", havingValue = "on")
public class FollowRecordTask {
    private final FollowRecordService followRecordService;
    private final FollowBizService followBizService;
    private final TodoRpcService todoRpcService;


    @Value("${follow.todo.FMCode}")
    @Getter
    private String FMCode;

    @Value("${follow.todo.RMCode}")
    @Getter
    private String RMCode;

    @Value("${follow.todo.IRCode}")
    @Getter
    private String IRCode;

    @Value("${follow.todo.ACCode}")
    @Getter
    private String ACCode;

    @Autowired
    public FollowRecordTask(final FollowRecordService followRecordService,
                            final FollowBizService followBizService,
                            final TodoRpcService todoRpcService) {
        this.followRecordService = followRecordService;
        this.followBizService = followBizService;
        this.todoRpcService = todoRpcService;
    }

    /**
     * 处理跟进逾期
     */
    @Scheduled(initialDelay = 1000 * 15, fixedRate = 1000 * 20)
    @Transactional(rollbackFor = Exception.class)
    public void endTaskRecord() {
        List<FollowRecord> list = followRecordService.list(Wrappers.<FollowRecord>lambdaQuery()
                .eq(FollowRecord::getOutTime, Boolean.FALSE)
                .le(FollowRecord::getDeadline, DateUtil.localDateTime2Date(LocalDateTime.now()))
                .isNull(FollowRecord::getFollowTime)
                .last("limit 0, 100")
        );
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        for (FollowRecord record : list) {
            followBizService.overdueProcessing(record);
        }
    }

    /**
     * 推送到待办系统
     */
    @Scheduled(cron = "0/8 * 5-22 * * *")
    @Transactional(rollbackFor = Exception.class)
    public void push2Todo() {
        List<FollowRecord> list = followRecordService.list(Wrappers.<FollowRecord>lambdaQuery()
                .eq(FollowRecord::getOutTime, Boolean.FALSE)
                .eq(FollowRecord::getAddTodo, Boolean.FALSE)
                .le(FollowRecord::getPlanTime, DateUtil.localDateTime2Date(LocalDateTime.now()))
                .last("limit 0,500")
        );
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        List<Long> idList = new ArrayList<>();
        for (FollowRecord record : list) {
            BackLogItemDTO dto = new BackLogItemDTO(record.getUserId(), getItemCode(record.getType()), String.valueOf(record.getId()), record.getPlanTime());
            boolean pushed = todoRpcService.push(dto);
            if (pushed) {
                idList.add(record.getId());
            }
        }
        followRecordService.addTodoBatchById(idList);
    }

    /**
     * 完成待办的重试
     */
    @Scheduled(initialDelay = 1500, fixedRate = 1000)
    public void retryCompleteTodoItem() {
        String key = todoRpcService.generateKey(TodoRpcService.TodoOperationEnum.COMPLETE);
        Optional<BackLogItemDTO> fromCache = todoRpcService.getOneFromCache(key);
        fromCache.ifPresent(todoRpcService::complete);
    }

    /**
     * 取消待办的重试
     */
    @Scheduled(initialDelay = 1000, fixedRate = 1000)
    public void retryCancelTodoItem() {
        String key = todoRpcService.generateKey(TodoRpcService.TodoOperationEnum.CANCEL);
        Optional<BackLogItemDTO> fromCache = todoRpcService.getOneFromCache(key);
        fromCache.ifPresent(todoRpcService::cancel);
    }

    private String getItemCode(FollowTypeEnum type) {
        switch (type) {
            case AC:
                return getACCode();
            case FM:
                return getFMCode();
            case IR:
                return getIRCode();
            case RM:
                return getRMCode();
            default:
                return getFMCode();
        }
    }

}
