package cn.fw.valhalla.service.bus;

import cn.fw.common.cache.locker.DistributedLocker;
import cn.fw.common.data.mybatis.pagination.PageData;
import cn.fw.common.page.AppPage;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.data.base.domain.common.Message;
import cn.fw.erp.sdk.api.result.UserRoleInfo;
import cn.fw.third.push.sdk.api.ImSendMessage;
import cn.fw.third.push.sdk.api.para.im.MsgPara;
import cn.fw.valhalla.common.constant.RoleCode;
import cn.fw.valhalla.common.enums.AllocationTypeEnum;
import cn.fw.valhalla.domain.db.LeaveNeedDo;
import cn.fw.valhalla.domain.db.customer.Customer;
import cn.fw.valhalla.domain.db.follow.FollowRecord;
import cn.fw.valhalla.domain.db.follow.FollowTask;
import cn.fw.valhalla.domain.dto.LeaveAllocationDTO;
import cn.fw.valhalla.domain.enums.FollowTypeEnum;
import cn.fw.valhalla.domain.enums.TaskStateEnum;
import cn.fw.valhalla.domain.query.LeaveQueryVO;
import cn.fw.valhalla.domain.vo.LeaveNeedDoVO;
import cn.fw.valhalla.rpc.erp.UserRoleRpcService;
import cn.fw.valhalla.service.data.CustomerService;
import cn.fw.valhalla.service.data.FollowRecordService;
import cn.fw.valhalla.service.data.FollowTaskService;
import cn.fw.valhalla.service.data.LeaveNeedDoService;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.tuple.Pair;
import org.redisson.api.RLock;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.BoundListOperations;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * @author : kurisu
 * @className : LeaveNeedDoBizService
 * @description : 人员变动处理
 * @date: 2020-10-16 17:04
 */
@Service
@Slf4j
public class LeaveNeedDoBizService {
    private final LeaveNeedDoService leaveNeedDoService;
    private final CustomerService customerService;
    private final UserRoleRpcService userRoleRpcService;
    private final FollowTaskService followTaskService;
    private final FollowRecordService followRecordService;
    private final DistributedLocker distributedLocker;
    private final ImSendMessage imSendMessage;
    /**
     * Redis工具
     */
    private final StringRedisTemplate redisTemplate;

    @Value("${spring.cache.custom.global-prefix}:LeaveNeedDo")
    @Getter
    private String keyPrefix;

    @Autowired
    public LeaveNeedDoBizService(final LeaveNeedDoService leaveNeedDoService,
                                 final CustomerService customerService,
                                 final UserRoleRpcService userRoleRpcService,
                                 final FollowTaskService followTaskService,
                                 final FollowRecordService followRecordService,
                                 final DistributedLocker distributedLocker,
                                 final ImSendMessage imSendMessage,
                                 final StringRedisTemplate redisTemplate) {
        this.leaveNeedDoService = leaveNeedDoService;
        this.customerService = customerService;
        this.userRoleRpcService = userRoleRpcService;
        this.followTaskService = followTaskService;
        this.followRecordService = followRecordService;
        this.distributedLocker = distributedLocker;
        this.imSendMessage = imSendMessage;
        this.redisTemplate = redisTemplate;
    }

    public AppPage<LeaveNeedDoVO> getList(LoginAuthBean currentUser, LeaveQueryVO queryVO) {
        List<Long> shopIds = userRoleRpcService.getManageShopIds(currentUser.getUserId(), RoleCode.BYKFP);
        BV.isFalse(CollectionUtils.isEmpty(shopIds), () -> "无权限操作，请检查角色权限是否正确");
        PageData<LeaveNeedDo> pageData = leaveNeedDoService.page(new PageData<>(queryVO), Wrappers.<LeaveNeedDo>lambdaQuery()
                .in(LeaveNeedDo::getShopId, shopIds)
                .eq(LeaveNeedDo::getDone, Boolean.FALSE)
                .eq(Objects.nonNull(queryVO.getType()), LeaveNeedDo::getType, queryVO.getType())
        );
        AppPage<LeaveNeedDoVO> page = AppPage.empty(queryVO);
        List<LeaveNeedDoVO> list = new ArrayList<>();
        List<LeaveNeedDo> records = Optional.ofNullable(pageData.getRecords()).orElse(new ArrayList<>());
        for (LeaveNeedDo needDo : records) {
            int i = customerService.count(Wrappers.<Customer>lambdaQuery()
                    .eq(Customer::getAdviserId, needDo.getUserId())
                    .eq(Customer::getYn, Boolean.TRUE)
            );
            list.add(LeaveNeedDoVO.with(needDo, i));
        }
        page.setData(list);
        return page;
    }

    @Transactional(rollbackFor = Exception.class)
    public void allocation(LoginAuthBean user, LeaveAllocationDTO dto) {
        final String key = generateKey(dto.getId());
        final String lockKey = getLockKey(dto.getId());
        Pair<Boolean, RLock> pair = distributedLocker.tryLock(lockKey, TimeUnit.SECONDS, 0, 30);
        BV.isTrue(Boolean.TRUE.equals(pair.getKey()), () -> "请勿重复提交");
        try {
            //准备分配 常规校验
            prepareAllocation(dto);
            //变更档案
            List<Customer> customerList = doAllocation(dto, key);
            //分配跟进任务
            dealFollowTask(customerList);
            //完成分配发送消息提醒等
            finish(user.getUserName(), dto.getId(), key);
        } catch (Exception e) {
            clearKey(key);
            distributedLocker.unlock(lockKey);
            throw e;
        }
    }

    private void prepareAllocation(LeaveAllocationDTO dto) {
        AllocationTypeEnum typeEnum = AllocationTypeEnum.ofValue(dto.getAllocationType());
        BV.notNull(typeEnum, () -> "分配方式不正确,请重试");
        dto.setType(typeEnum);
        LeaveNeedDo needDo = leaveNeedDoService.queryProcessableById(dto.getId());
        BV.notNull(needDo, () -> "该条记录已处理或不存在，请刷新后重试");
        dto.setAdviserId(needDo.getUserId());
        if (AllocationTypeEnum.ONE.equals(typeEnum)) {
            BV.notNull(dto.getUserId(), () -> "指定人员不能为空");
            List<Long> shopIdList = userRoleRpcService.getManageShopIds(dto.getUserId(), RoleCode.FWGW);
            BV.isNotEmpty(shopIdList, () -> "指定人员非服务顾问角色，请核对");
            Long shopId = shopIdList.get(0);
            BV.isTrue(needDo.getShopId().equals(shopId), () -> "指定人员所属门店与档案归属门店不符");
            dto.setShopId(shopId);
        }

        if (AllocationTypeEnum.ALL.equals(dto.getType())) {
            dto.setShopId(needDo.getShopId());
        }
    }

    /**
     * 处理数据
     *
     * @param dto
     * @return 处理完的档案
     */
    private List<Customer> doAllocation(LeaveAllocationDTO dto, String key) {
        List<Customer> customerList = customerService.queryByAdviserId(dto.getAdviserId());
        if (CollectionUtils.isEmpty(customerList)) {
            return null;
        }
        if (AllocationTypeEnum.ONE.equals(dto.getType())) {
            allocation(key, customerList, dto);
        }
        if (AllocationTypeEnum.ALL.equals(dto.getType())) {
            allocation(key, customerList, dto.getShopId());
        }
        return customerList;
    }

    /**
     * 分配给指定人员
     *
     * @param list
     * @param dto
     */
    private void allocation(String key, List<Customer> list, LeaveAllocationDTO dto) {
        for (Customer customer : list) {
            customer.setShopId(dto.getShopId());
            customer.setAdviserId(dto.getUserId());
        }
        customerService.updateBatchById(list);
        setToCache(key, new UserInfo(dto.getUserId(), "", list.size()));
    }

    /**
     * 门店内平均分配
     *
     * @param list
     * @param shopId
     */
    private void allocation(String key, List<Customer> list, Long shopId) {
        List<UserRoleInfo> users = userRoleRpcService.getUsers(shopId, RoleCode.FWGW);
        BV.isNotEmpty(users, () -> "该门店没有服务顾问，请检查配置是否正确");
        LinkedList<UserInfo> queue = new LinkedList<>();
        for (UserRoleInfo user : users) {
            queue.offer(new UserInfo(user.getUserId(), user.getUserName()));
        }
        for (Customer customer : list) {
            UserInfo info = queue.poll();
            customer.setShopId(shopId);
            customer.setAdviserId(Objects.requireNonNull(info, "服务顾问信息获取异常，请重试").getUserId());
            info.setCount(info.getCount() + 1);
            queue.offer(info);
        }
        customerService.updateBatchById(list);
        setToCache(key, queue);
    }

    /**
     * 完成分配的后续处理逻辑
     *
     * @param list
     */
    private void dealFollowTask(List<Customer> list) {
        List<FollowTypeEnum> enumList = Arrays.asList(FollowTypeEnum.FM, FollowTypeEnum.RM);
        if (!CollectionUtils.isEmpty(list)) {
            //处理待办等信息
            for (Customer customer : list) {
                final Long customerId = customer.getId();
                final Long shopId = customer.getShopId();
                final Long adviserId = customer.getAdviserId();
                List<FollowTask> taskList = followTaskService.getListByCustomerId(customerId, enumList);
                taskList.forEach(task -> {
                    task.setFollowUser(adviserId);
                    task.setFollowShop(shopId);
                    if (TaskStateEnum.WAITING.equals(task.getState())) {
                        task.setOriginUser(adviserId);
                        task.setOriginShop(shopId);
                    } else {
                        task.setChanged(Boolean.TRUE);
                    }
                });
                List<FollowRecord> recordList = followRecordService.getRecordListByCustomer(customerId, enumList);
                recordList.forEach(record -> {
                    record.setUserId(adviserId);
                    record.setShopId(shopId);
                });
                if (taskList.size() > 0) {
                    followTaskService.updateBatchById(taskList);
                }
                if (recordList.size() > 0) {
                    followRecordService.updateBatchById(recordList);
                }
            }
        }
    }

    private void finish(String userName, Long leaveId, String key) {
        leaveNeedDoService.dealById(leaveId);
        List<UserInfo> list = getAllFromCache(key);
        for (UserInfo info : list) {
            try {
                String text = String.format("%s通过资源分配给你%s台保有客", userName, info.getCount());
                final MsgPara msgPara = MsgPara.getOfflineTxetPara(text, null, "保有客分配",
                        "保有客分配", null, info.getUserId()).build();
                final Message<Integer> msg = imSendMessage.sendMsg(msgPara);
                log.info("给[{}]推送im消息结果:[{}]", info, msg.getResult());
            } catch (Exception e) {
                log.error("给[{}]推送im消息失败：]", info, e);
            }
        }
        clearKey(key);
    }

    private String getLockKey(final Long leaveId) {
        BV.notNull(leaveId, "leaveId cannot be null");

        return String.format("%s:lock:%s", getKeyPrefix(), leaveId);
    }

    private String generateKey(final Long leaveId) {
        Assert.notNull(leaveId, "leaveId cannot be null");

        return String.format("%s:message:%s", getKeyPrefix(), leaveId);
    }

    public List<UserInfo> getAllFromCache(final String key) {
        try {
            Boolean hasKey = redisTemplate.hasKey(key);
            if (!Boolean.TRUE.equals(hasKey)) {
                return new ArrayList<>();
            }
            BoundListOperations<String, String> ops = redisTemplate.boundListOps(key);
            List<String> stringList = ops.range(0, -1);
            if (CollectionUtils.isEmpty(stringList)) {
                return new ArrayList<>();
            }
            List<UserInfo> dtos = stringList.stream().map(str -> JSONObject.parseObject(str, UserInfo.class)).collect(Collectors.toList());
            redisTemplate.delete(key);
            return dtos;
        } catch (Exception e) {
            log.error("缓存设置信息失败 key[{}]", key, e);
            return new ArrayList<>();
        }
    }

    private void setToCache(final String key, final UserInfo userInfo) {
        try {
            BoundListOperations<String, String> ops = redisTemplate.boundListOps(key);
            String jsonString = JSONObject.toJSONString(userInfo);
            ops.rightPush(jsonString);
        } catch (Exception e) {
            log.error("缓存设置信息失败 key[{}]", key, e);
        }
    }

    private void setToCache(final String key, final LinkedList<UserInfo> userInfos) {
        try {
            if (CollectionUtils.isEmpty(userInfos)) {
                return;
            }
            BoundListOperations<String, String> ops = redisTemplate.boundListOps(key);
            String[] users = userInfos.stream().map(JSONObject::toJSONString).toArray(String[]::new);
            ops.rightPushAll(users);
        } catch (Exception e) {
            log.error("缓存设置信息失败 key[{}]", key, e);
        }
    }

    private void clearKey(final String key) {
        try {
            redisTemplate.delete(key);
        } catch (Exception e) {
            log.error("清空缓存信息失败 key[{}]", key, e);
        }
    }

    static class UserInfo {
        private Long userId;
        private String userName;
        private int count;

        public UserInfo() {
        }

        public UserInfo(Long userId, String userName) {
            this.userId = userId;
            this.userName = userName;
        }

        public UserInfo(Long userId, String userName, int count) {
            this.userId = userId;
            this.userName = userName;
            this.count = count;
        }

        public Long getUserId() {
            return userId;
        }

        public void setUserId(Long userId) {
            this.userId = userId;
        }

        public String getUserName() {
            return userName;
        }

        public void setUserName(String userName) {
            this.userName = userName;
        }

        public int getCount() {
            return count;
        }

        public void setCount(int count) {
            this.count = count;
        }

        @Override
        public String toString() {
            return "UserInfo{" +
                    "userId=" + userId +
                    ", userName='" + userName + '\'' +
                    ", count=" + count +
                    '}';
        }
    }
}
