package cn.fw.valhalla.service.bus.follow;

import cn.fw.valhalla.common.utils.DateUtil;
import cn.fw.valhalla.common.utils.StringUtils;
import cn.fw.valhalla.domain.db.customer.Customer;
import cn.fw.valhalla.domain.db.customer.CustomerBaseInfo;
import cn.fw.valhalla.domain.db.follow.FollowNoticeRecord;
import cn.fw.valhalla.domain.db.follow.FollowTask;
import cn.fw.valhalla.domain.enums.FollowTypeEnum;
import cn.fw.valhalla.domain.enums.SendStatusEnum;
import cn.fw.valhalla.domain.enums.SettingTypeEnum;
import cn.fw.valhalla.domain.enums.SettingUnitEnum;
import cn.fw.valhalla.domain.vo.setting.SettingVO;
import cn.fw.valhalla.rpc.angel.InsurerRpcService;
import cn.fw.valhalla.rpc.angel.dto.InsuranceDTO;
import cn.fw.valhalla.rpc.oop.OopService;
import cn.fw.valhalla.rpc.oop.dto.ShopDTO;
import cn.fw.valhalla.rpc.passport.TemplateMessageService;
import cn.fw.valhalla.rpc.passport.dto.RenewInsurNoticeMessageParam;
import cn.fw.valhalla.service.bus.setting.SettingBizService;
import cn.fw.valhalla.service.data.CustomerBaseInfoService;
import cn.fw.valhalla.service.data.CustomerService;
import cn.fw.valhalla.service.data.FollowNoticeRecordService;
import cn.fw.valhalla.service.data.FollowTaskService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.sql.Timestamp;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

import static cn.fw.valhalla.service.bus.follow.strategy.AbstractFollowStrategy.getCalendarType;

/**
 * @author : kurisu
 * @className : SendNoticeService
 * @description : 发送消息
 * @date: 2020-08-26 15:48
 */
@Service
@Slf4j
public class NoticeBizService {
    private final TemplateMessageService templateMessageService;
    private final FollowNoticeRecordService followNoticeRecordService;
    private final FollowTaskService followTaskService;
    private final SettingBizService settingBizService;
    private final InsurerRpcService insurerRpcService;
    private final CustomerService customerService;
    private final CustomerBaseInfoService customerBaseInfoService;
    private final OopService oopService;

    private final String type = "d";
    private final int day = 2;

    @Autowired
    public NoticeBizService(final TemplateMessageService templateMessageService,
                            final FollowNoticeRecordService followNoticeRecordService,
                            final FollowTaskService followTaskService,
                            final SettingBizService settingBizService,
                            final InsurerRpcService insurerRpcService,
                            final CustomerService customerService,
                            final CustomerBaseInfoService customerBaseInfoService,
                            final OopService oopService) {
        this.templateMessageService = templateMessageService;
        this.followNoticeRecordService = followNoticeRecordService;
        this.followTaskService = followTaskService;
        this.settingBizService = settingBizService;
        this.insurerRpcService = insurerRpcService;
        this.customerService = customerService;
        this.customerBaseInfoService = customerBaseInfoService;
        this.oopService = oopService;
    }

    /**
     * 发送模板消息
     */
    public void sendNotice() {
        List<FollowNoticeRecord> list = followNoticeRecordService.list(Wrappers.<FollowNoticeRecord>lambdaQuery()
                .eq(FollowNoticeRecord::getStatus, SendStatusEnum.NOT_SENT)
                .le(FollowNoticeRecord::getSendTime, DateUtil.localDateTime2Date(LocalDateTime.now()))
                .last("limit 0, 100")
        );
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        for (FollowNoticeRecord record : list) {
            sendNoticeAndCreate(record);
        }
    }

    /**
     * 推送模板消息 完成后生成下一次推送
     *
     * @param record
     */
    private void sendNoticeAndCreate(FollowNoticeRecord record) {
        FollowTask task = followTaskService.getById(record.getTaskId());
        if (Objects.isNull(task)) {
            return;
        }
        Customer customer = customerService.queryById(task.getCustomerId());
        if (Objects.isNull(customer)) {
            createNextNotice(record, task.getGroupId());
            return;
        }
        CustomerBaseInfo baseInfo = customerBaseInfoService.queryById(customer.getBaseId());
        if (Objects.isNull(baseInfo)) {
            createNextNotice(record, task.getGroupId());
            return;
        }
        ShopDTO shop = oopService.shop(customer.getShopId());
        String shopName = Objects.isNull(shop) ? "" : shop.getShopName();
        boolean flag = false;
        if (FollowTypeEnum.IR.equals(record.getFollowType())) {
            flag = sendIR(record, task, baseInfo.getMemberId(), shopName);
        }
        if (FollowTypeEnum.FM.equals(record.getFollowType())) {
            flag = sendFM(record, task, baseInfo.getMemberId(), shopName);
        }
        if (FollowTypeEnum.RM.equals(record.getFollowType())) {
            flag = sendRM(record, task, baseInfo.getMemberId(), shopName);
        }
        if (flag) {
            createNextNotice(record, task.getGroupId());
        }
    }

    private boolean sendIR(final FollowNoticeRecord record, final FollowTask task, Long memberId, String shopName) {
        if (DateUtil.sub(new Date(), record.getSendTime(), type) > day) {
            record.setStatus(SendStatusEnum.SUCCESS);
            return followNoticeRecordService.updateById(record);
        }
        RenewInsurNoticeMessageParam param = new RenewInsurNoticeMessageParam();
        param.setMemberId(memberId);
        param.setShopName(shopName);
        param.setExpireDate(DateUtil.getFullDateString(task.getOriginTime()));
        InsuranceDTO dto = insurerRpcService.queryInsByCustId(task.getCustomerId());
        if (Objects.nonNull(dto)) {
            param.setInsurCompanyName(dto.getInsurerName());
        }
        String result = templateMessageService.sendRINotice(param);
        record.setStatus(StringUtils.isEmpty(result) ? SendStatusEnum.SUCCESS : SendStatusEnum.FAILED);
        record.setStatusDesc(result);
        return followNoticeRecordService.updateById(record);
    }

    private boolean sendFM(final FollowNoticeRecord record, final FollowTask task, Long memberId, String shopName) {
        if (DateUtil.sub(new Date(), record.getSendTime(), type) > day) {
            record.setStatus(SendStatusEnum.SUCCESS);
            return followNoticeRecordService.updateById(record);
        }
        RenewInsurNoticeMessageParam param = new RenewInsurNoticeMessageParam();
        param.setMemberId(memberId);
        param.setShopName(shopName);
        param.setExpireDate(DateUtil.getFullDateString(task.getDeadline()));

        String result = templateMessageService.sendFMNotice(param);
        record.setStatus(StringUtils.isEmpty(result) ? SendStatusEnum.SUCCESS : SendStatusEnum.FAILED);
        record.setStatusDesc(result);
        return followNoticeRecordService.updateById(record);
    }

    private boolean sendRM(final FollowNoticeRecord record, final FollowTask task, Long memberId, String shopName) {
        if (DateUtil.sub(new Date(), record.getSendTime(), type) > day) {
            record.setStatus(SendStatusEnum.SUCCESS);
            return followNoticeRecordService.updateById(record);
        }
        RenewInsurNoticeMessageParam param = new RenewInsurNoticeMessageParam();
        param.setMemberId(memberId);
        param.setShopName(shopName);
        param.setExpireDate(DateUtil.getFullDateString(task.getDeadline()));

        String result = templateMessageService.sendRMNotice(param);
        record.setStatus(StringUtils.isEmpty(result) ? SendStatusEnum.SUCCESS : SendStatusEnum.FAILED);
        record.setStatusDesc(result);
        return followNoticeRecordService.updateById(record);
    }

    private void createNextNotice(final FollowNoticeRecord record, Long groupId) {
        Optional<SettingVO> timesSetting = settingBizService.querySettingByType(record.getFollowType(), SettingTypeEnum.NOTICE_TIMES, groupId);
        if (!timesSetting.isPresent()) {
            return;
        }
        int count = followNoticeRecordService.count(Wrappers.<FollowNoticeRecord>lambdaQuery().eq(FollowNoticeRecord::getTaskId, record.getTaskId()));
        int value = Optional.ofNullable(timesSetting.get().getDetailValue()).orElse(0);
        if (count >= value) {
            return;
        }
        Optional<SettingVO> cycleSetting = settingBizService.querySettingByType(record.getFollowType(), SettingTypeEnum.NOTICE_CYCLE, groupId);
        cycleSetting.ifPresent(r -> {
            Timestamp expired = DateUtil.getExpired(record.getSendTime(), r.getDetailValue(), getCalendarType(Objects.requireNonNull(SettingUnitEnum.ofValue(r.getUnit()))));
            followNoticeRecordService.save(createEntity(record, expired));
        });
    }

    private FollowNoticeRecord createEntity(final FollowNoticeRecord record, Timestamp time) {
        FollowNoticeRecord noticeRecord = new FollowNoticeRecord();
        noticeRecord.setCustomerId(record.getCustomerId());
        noticeRecord.setTaskId(record.getTaskId());
        noticeRecord.setFollowType(record.getFollowType());
        noticeRecord.setStatus(SendStatusEnum.NOT_SENT);
        noticeRecord.setSendTime(time);
        noticeRecord.setCreateTime(DateUtil.localDateTime2Date(LocalDateTime.now()));
        return noticeRecord;
    }
}
