package cn.fw.valhalla.service.bus.follow.strategy.impl;

import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.valhalla.common.constant.RoleCode;
import cn.fw.valhalla.common.utils.DateUtil;
import cn.fw.valhalla.common.utils.MobileUtil;
import cn.fw.valhalla.domain.db.OriginalData;
import cn.fw.valhalla.domain.db.customer.Customer;
import cn.fw.valhalla.domain.db.follow.FollowNoticeRecord;
import cn.fw.valhalla.domain.db.follow.FollowRecord;
import cn.fw.valhalla.domain.db.follow.FollowTask;
import cn.fw.valhalla.domain.dto.CustomerDetailDto;
import cn.fw.valhalla.domain.dto.FollowAttachmentDTO;
import cn.fw.valhalla.domain.enums.*;
import cn.fw.valhalla.domain.vo.follow.*;
import cn.fw.valhalla.domain.vo.setting.SettingVO;
import cn.fw.valhalla.rpc.angel.dto.InsuranceDTO;
import cn.fw.valhalla.rpc.erp.dto.PostUserDTO;
import cn.fw.valhalla.rpc.erp.dto.UserRoleDataRangeDTO;
import cn.fw.valhalla.rpc.oop.dto.ShopDTO;
import cn.fw.valhalla.service.bus.follow.strategy.AbstractFollowStrategy;
import cn.fw.valhalla.service.event.CancelApproveEvent;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.sql.Timestamp;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * @author : kurisu
 * @className : IRFollowStrategy
 * @description : 续保策略
 * @date: 2020-08-17 10:48
 */
@Slf4j
@Component
public class IRFollowStrategy extends AbstractFollowStrategy {
    @Override
    public FollowTypeEnum getFollowType() {
        return FollowTypeEnum.IR;
    }

    @Override
    public List<FollowTodoListVO> getList(Integer startIndex, Integer pageSize, Long userId) {
        List<FollowRecord> list = list(startIndex, pageSize, userId, getFollowType());
        if (CollectionUtils.isEmpty(list)) {
            return new ArrayList<>();
        }
        List<FollowTodoListVO> voList = new ArrayList<>();
        for (FollowRecord followRecord : list) {
            IRTodoListVO vo = new IRTodoListVO();
            vo.setId(followRecord.getId());
            vo.setTaskId(followRecord.getTaskId());
            vo.setCustomerId(followRecord.getCustomerId());
            vo.setDeadline(followRecord.getDeadline());
            CustomerDetailDto customerDetailDto = queryCustomerInfo(followRecord.getCustomerId());
            vo.setName(customerDetailDto.getName());
            vo.setCarImage(customerDetailDto.getCarImage());
            Optional<InsuranceDTO> insuranceDTO = queryInsuInfo(followRecord.getCustomerId());
            insuranceDTO.ifPresent(ins -> {
                vo.setInsComName(ins.getInsurerName());
                vo.setInsExpiration(ins.getExpiryDate());
            });
            voList.add(vo);
        }
        return voList;
    }

    @Override
    public FollowDetailVO getDetail(Long id) {
        FollowRecord followRecord = followRecordService.getById(id);
        BV.notNull(followRecord, "跟进记录不存在");
        IRDetailVO vo = assemble(followRecord.getCustomerId());
        vo.setId(followRecord.getId());
        vo.setTaskId(followRecord.getTaskId());
        vo.setDeadline(followRecord.getDeadline());
        return vo;
    }

    @Override
    public List<FollowRecordVO> getRecordList(Long taskId) {
        return super.getRecordList(taskId);
    }

    @Override
    public void completeRecord(Long id, LoginAuthBean currentUser) {
        super.completeRecord(id, currentUser);
    }

    @Override
    public void uploadAtt(FollowAttachmentDTO dto, Long userId) {
        super.uploadAtt(dto, userId);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void cancelFollowTask(Long customerId, Date time, Long groupId) {
        List<FollowTask> list = followTaskService.list(Wrappers.<FollowTask>lambdaQuery()
                .eq(FollowTask::getCustomerId, customerId)
                .eq(FollowTask::getGroupId, groupId)
                .eq(FollowTask::getType, getFollowType())
                .and(w -> w.ge(FollowTask::getCreateTime, time).or().ge(FollowTask::getFinishTime, time))
        );

        for (FollowTask followTask : list) {
            if (TaskStateEnum.WAITING.equals(followTask.getState())) {
                followTaskService.removeById(followTask.getId());
                followNoticeRecordService.removeByTaskId(followTask.getId());
            }
            if (TaskStateEnum.ONGOING.equals(followTask.getState())) {
                followTaskService.removeById(followTask.getId());
                followNoticeRecordService.removeByTaskId(followTask.getId());
                cancelFollowTodo(followTask.getId());
            }
            if (TaskStateEnum.END.equals(followTask.getState())) {
                rollback(followTask);
            }
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void settingChanged(List<SettingVO> setting, Long groupId) {
        List<FollowTask> list = followTaskService.list(Wrappers.<FollowTask>lambdaQuery()
                .eq(FollowTask::getState, TaskStateEnum.WAITING)
                .eq(FollowTask::getType, getFollowType())
        );
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        updateTask(list, setting);

        SettingVO noticeSetting = setting.stream().filter(r -> SettingTypeEnum.FIRST_NOTICE_TIME.getValue().equals(r.getType())).findFirst().orElse(new SettingVO());
        updateNoticeRecord(list, noticeSetting);
    }

    @Override
    protected void updateTask(List<FollowTask> list, List<SettingVO> setting) {
        for (SettingVO vo : setting) {
            final Integer unit = vo.getUnit();
            final int value = Optional.ofNullable(vo.getDetailValue()).orElse(0);
            SettingUnitEnum unitEnum = SettingUnitEnum.ofValue(unit);

            if (SettingTypeEnum.FIRST_TRIGGER_TIME.getValue().equals(vo.getType())) {
                if (Objects.nonNull(unitEnum) && value > 0) {
                    final int calendarType = getCalendarType(unitEnum);
                    list.forEach(task -> {
                        Date originTime = task.getOriginTime();
                        Timestamp timestamp = DateUtil.getExpired(originTime, -1 * value, calendarType);
                        task.setBeginTime(timestamp);
                    });
                }
            }
            if (SettingTypeEnum.CHANGE_TIME.getValue().equals(vo.getType())) {
                if (Objects.nonNull(unitEnum) && value > 0) {
                    final int calendarType = getCalendarType(unitEnum);
                    list.forEach(task -> {
                        Date originTime = task.getOriginTime();
                        Timestamp timestamp = DateUtil.getExpired(originTime, -1 * value, calendarType);
                        task.setBeginTime(timestamp);
                    });
                }
            }
            if (SettingTypeEnum.FAIL_TIME.getValue().equals(vo.getType())) {
                if (Objects.nonNull(unitEnum) && value > 0) {
                    final int calendarType = getCalendarType(unitEnum);
                    list.forEach(task -> {
                        Date originTime = task.getOriginTime();
                        Timestamp timestamp = DateUtil.getExpired(originTime, value, calendarType);
                        task.setBeginTime(timestamp);
                    });
                }
            }
        }
        followTaskService.updateBatchById(list);
    }

    /**
     * 更新服务号消息记录
     *
     * @param list
     * @param vo
     */
    @Override
    protected void updateNoticeRecord(List<FollowTask> list, SettingVO vo) {
        List<Long> idList = list.stream().map(FollowTask::getId).collect(Collectors.toList());
        List<FollowNoticeRecord> noticeList = followNoticeRecordService.list(Wrappers.<FollowNoticeRecord>lambdaQuery()
                .eq(FollowNoticeRecord::getStatus, SendStatusEnum.NOT_SENT)
                .in(FollowNoticeRecord::getTaskId, idList)
        );
        Integer unit = vo.getUnit();
        int value = Optional.ofNullable(vo.getDetailValue()).orElse(0);
        SettingUnitEnum unitEnum = SettingUnitEnum.ofValue(unit);
        if (Objects.isNull(unitEnum) || value <= 0) {
            return;
        }
        final int calendarType = getCalendarType(unitEnum);
        for (FollowTask task : list) {
            for (FollowNoticeRecord noticeRecord : noticeList) {
                if (task.getId().equals(noticeRecord.getTaskId())) {
                    Date originTime = task.getOriginTime();
                    Timestamp timestamp = DateUtil.getExpired(originTime, -1 * value, calendarType);
                    noticeRecord.setSendTime(timestamp);
                }
            }
        }
        followNoticeRecordService.updateBatchById(noticeList);
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean origin2task(OriginalData originalData) {
        Customer customer = customerService.queryById(originalData.getCustomerId());
        if (Objects.isNull(customer)) {
            return true;
        }
        this.completeTask(customer, originalData.getGenerateTime());

        final FollowTask task = createTask(originalData, getFollowType());
        if (Objects.isNull(task)) {
            return true;
        }
        Long shopId = customer.getShopId();
        List<PostUserDTO> userList = userService.getUserByRole(shopId, RoleCode.XBGJ);

        if (CollectionUtils.isEmpty(userList)) {
            log.error("跟进人员获取失败：没有续保跟进员");
            return false;
        }
        task.setOriginUser(userList.get(0).getUserId());
        task.setFollowUser(userList.get(0).getUserId());

        userList.stream().filter(r -> customer.getAdviserId().equals(r.getUserId())).findFirst().ifPresent(user -> {
            task.setOriginUser(user.getUserId());
            task.setFollowUser(user.getUserId());
        });

        ShopDTO shop = oopService.shop(shopId);
        if (Objects.nonNull(shop)) {
            task.setDealerId(shop.getDealerId());
        }
        task.setOriginShop(shopId);
        task.setFollowShop(shopId);
        boolean saved = followTaskService.save(task);
        if (!saved) {
            return true;
        }
        return this.createFirstNotice(task);
    }

    /**
     * 生成任务实体
     *
     * @param originalData
     * @return
     */
    @Override
    protected FollowTask createTask(final OriginalData originalData, FollowTypeEnum followType) {
        final FollowTask followTask = new FollowTask();
        followTask.setCustomerId(originalData.getCustomerId());
        followTask.setType(followType);
        followTask.setState(TaskStateEnum.WAITING);
        Optional<InsuranceDTO> optional = insurerRpcService.getByInsId(originalData.getCustomerId(), originalData.getDetailId());
        if (optional.isPresent()) {
            followTask.setOriginTime(optional.get().getExpiryDate());
        } else {
            followTask.setOriginTime(DateUtil.getExpired(originalData.getGenerateTime(), 1, Calendar.YEAR));
        }
        settingBizService.querySettingByType(followType, SettingTypeEnum.FIRST_TRIGGER_TIME, originalData.getGroupId())
                .ifPresent(r -> followTask.setBeginTime(calDate(r, followTask.getOriginTime(), true)));

        settingBizService.querySettingByType(followType, SettingTypeEnum.CHANGE_TIME, originalData.getGroupId())
                .ifPresent(r -> followTask.setChangeUserTime(calDate(r, followTask.getOriginTime(), true)));

        settingBizService.querySettingByType(followType, SettingTypeEnum.FAIL_TIME, originalData.getGroupId())
                .ifPresent(r -> followTask.setDeadline(calDate(r, followTask.getOriginTime(), false)));
        followTask.setFinished(Boolean.FALSE);
        followTask.setGroupId(originalData.getGroupId());
        followTask.setCreateTime(originalData.getGenerateTime());

        return followTask;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long createFirstRecord(FollowTask task) {
        return super.createFirstRecord(task);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void overdueProcessing(FollowRecord record) {
        super.overdueProcessing(record);
    }

    @Override
    public FollowDetailVO followPoolDetail(FollowTask task) {
        IRDetailVO vo = assemble(task.getCustomerId());
        vo.setTaskId(task.getId());
        return vo;
    }

    /**
     * 生成消息推送
     *
     * @param task
     */
    @Override
    protected boolean createFirstNotice(final FollowTask task) {
        if (!task.getDeadline().after(task.getBeginTime()) || !task.getDeadline().after(DateUtil.localDateTime2Date(LocalDateTime.now()))) {
            return true;
        }
        final FollowNoticeRecord record = new FollowNoticeRecord();
        record.setTaskId(task.getId());
        record.setCustomerId(task.getCustomerId());
        record.setStatus(SendStatusEnum.NOT_SENT);
        settingBizService.querySettingByType(task.getType(), SettingTypeEnum.FIRST_NOTICE_TIME, task.getGroupId())
                .ifPresent(r -> record.setSendTime(calDate(r, task.getOriginTime(), true)));
        record.setCreateTime(new Date());
        record.setFollowType(task.getType());
        return followNoticeRecordService.save(record);
    }


    /**
     * 完成之前的跟进
     *
     * @param customer
     * @param completeTime
     */
    private void completeTask(Customer customer, Date completeTime) {
        List<FollowTask> list = followTaskService.list(Wrappers.<FollowTask>lambdaQuery()
                .eq(FollowTask::getType, getFollowType())
                .ne(FollowTask::getState, TaskStateEnum.END)
                .eq(FollowTask::getCustomerId, customer.getId())
        );
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        List<Long> idList = new ArrayList<>();
        List<FollowTask> taskList = new ArrayList<>();
        for (FollowTask task : list) {
            if (TaskStateEnum.WAITING.equals(task.getState())) {
                idList.add(task.getId());
            }

            if (TaskStateEnum.ONGOING.equals(task.getState())) {
                List<UserRoleDataRangeDTO> range = userService.getUserRoleDataRange(task.getFollowUser(), RoleCode.FWGW);
                task.setFinishUser(customer.getAdviserId());
                task.setFinishShop(customer.getShopId());
                if (CollectionUtils.isEmpty(range)) {
                    task.setFinishUser(task.getFollowUser());
                    task.setFinishShop(task.getFollowShop());
                }
                task.setFinished(Boolean.TRUE);
                task.setState(TaskStateEnum.END);
                task.setFinishTime(completeTime);
                completeTodo(task.getId());
                taskList.add(task);
                CancelApproveEvent event = new CancelApproveEvent(task.getId(), ApproveTypeEnum.FOLLOW_DEFEAT);
                eventPublisher.publishEvent(event);
            }
        }
        List<Long> ids = list.stream().map(FollowTask::getId).collect(Collectors.toList());
        followNoticeRecordService.removeByTaskIds(ids);
        if (idList.size() > 0) {
            followTaskService.removeByIds(idList);
        }
        if (taskList.size() > 0) {
            followTaskService.updateBatchById(taskList);
        }
    }

    @Override
    public IRDetailVO assemble(Long customerId) {
        CustomerDetailDto detailDto = queryCustomerInfo(customerId);
        IRDetailVO vo = new IRDetailVO();
        vo.setCustomerId(customerId);
        vo.setAdviserId(detailDto.getAdviserId());
        vo.setAdviserName(detailDto.getAdviserName());
        vo.setVin(detailDto.getFrameNo());
        vo.setName(detailDto.getName());
        vo.setMobile(detailDto.getMobile());
        vo.setRegion(MobileUtil.attribution(detailDto.getMobile()));
        vo.setRealMobile(detailDto.getMobile());
        vo.setPlateNo(detailDto.getPlateNo());
        vo.setCarModel(detailDto.getBrandName() + " " + detailDto.getSeriesName());
        Optional<InsuranceDTO> insuranceDTO = queryInsuInfo(customerId);
        insuranceDTO.ifPresent(ins -> {
            vo.setTclInsComName(ins.getTciInsurerName());
            vo.setTclInsExpiration(ins.getTciExpiryDate());
            vo.setBusInsComName(ins.getInsurerName());
            vo.setBusInsExpiration(ins.getExpiryDate());
        });
        return vo;
    }
}
